@@ -4,7 +4,7 @@ test_that("MCMC Monthly, in_type=='params', distr='poisson'", {
   base_forecasts_in = read.csv(file = "dataForTests/Monthly-Poisson_basef.csv", header = FALSE)
   base_forecasts = list()
   for (i in 1:nrow(base_forecasts_in)) {
-    base_forecasts[[i]] = list(as.numeric(base_forecasts_in[i,]))[[1]]
+    base_forecasts[[i]] = list(lambda = base_forecasts_in[i,1])
   }
   res.buis = reconc_BUIS(S, base_forecasts,
                      in_type = "params", distr = "poisson", num_samples = 100000,seed=42)
